import React, { useEffect, useState } from 'react'
import { Form, Space, TreeSelect, Button, Input, Modal, message, Select } from 'antd'
import { MinusCircleOutlined, PlusOutlined } from '@ant-design/icons'
import styles from './index.less'
import {
  LoadLedgers,
  ReloadTableFields,
  QueryFields,
  getMaplayerByTerminalType,
  GetLayerList,
  GetLayerFields,
  GetLayersShareField,
  GetAreaTaskProgramme
} from '../../../../../apis/process'
import Drag from '../../../../components/Drag'

const { TreeNode } = TreeSelect

const initField = [
  { fromField: '工单编号', toField: '工单编号' },
  { fromField: 'GIS图层', toField: 'GIS图层' },
  { fromField: 'GIS编号', toField: 'GIS编号' },
]

const AreaTaskShine = (props) => {

  const { addons, onChange, value , schema } = props
  const formData = addons.formData
  const {title} = schema;

  const [form] = Form.useForm()
  const [visible, setVisible] = useState(false)
  const [tableName, setTableName] = useState(formData?.tableName)
  const [accountName, setAccountName] = useState(formData?.accountName)
  const [taskList, setTaskList] = useState([])
  const [accountList, setAccountList] = useState([])
  const [mapServers, setMapServers] = useState([])
  const [service, setService] = useState(formData?.service)
  const [layers, setLayers] = useState([])
  const [layerName, setlayerName] = useState(formData?.layerName)
  const [fromField, setFromField] = useState([])
  const [toField, setToField] = useState([])

  const onOk = () => {
    form
      .validateFields()
      .then((values) => {
        const { field, schemeName, accountName, service, layerName } = form.getFieldsValue()
        // if (!toField.some(v => v.name === '工单编号')) {
        //   return message.error(`【${accountName}】台账不存在工单编号字段`)
        // }
        // if (!toField.some(v => v.name === '反馈状态')) {
        //   return message.error(`【${accountName}】台账不存在反馈状态字段`)
        // }
        addons.setValue('schemeName', schemeName)
        addons.setValue('tableName', tableName)
        addons.setValue('accountName', accountName)
        addons.setValue('service', service)
        addons.setValue('layerName', layerName)
        onChange(field)
        setVisible(false)
      })
      .catch((info) => {
        console.log('Validate Failed:', info)
      })
  }

  const inputClick = async () => {
    form.setFieldsValue({
      field: value.length ? value : initField,
      accountName: formData?.accountName,
      service: formData?.service,
      layerName: formData?.layerName || []
    })
    getData()
    setAccountName(formData?.accountName)
    setService(formData?.service)
    setlayerName(formData?.layerName)
    setVisible(true)
  }

  const getData = () => {
    getAccountList()
    getServices()
  }

  const getAccountList = async () => {
    const { code, data } = await LoadLedgers('GIS反馈')
    if (code === 0) {
      setAccountList(data?.root)
    }
  }

  const getServices = async () => {
    const { code, data } = await getMaplayerByTerminalType()
    if (code === '0') {
      setMapServers(data?.map(v => ({ label: v.schemename, value: v.servicename })))
    }
  }

  const layerNameFocus = async () => {
    let { success, root } = await GetLayerList(service)
    if (success === true) {
      let array = []
      root.forEach(v => {
        let values = v.layerName.split(':')
        if (values.length === 2) {
          array.push({ layerID: v.layerID, layerName: values[1] })
        }
      })
      setLayers(array)
    }
  }

  const getToField = async () => {
    const { code, data } = await ReloadTableFields({ tableName: tableName })
    if (code === 0) {
      setToField(data?.root)
    }
  }

  const getFromField = async (value) => {
    const { code, data } = await GetLayersShareField({ mapServer: service, layerNames: layerName })
    if (code === '0') {
      setFromField(data || [])
    }
  }

  const fromFieldChange = (value) => {
    const { field, service, layerName } = form.getFieldsValue()
    let values = field.map(v => ({ ...v, layerName: v.fromField === value ? layerName : v.layerName }))
    form.setFieldsValue({ field: values, service, layerName })
  }

  const taskNameChange = (value) => {
    let item = taskList.find(v => v.Name === value)
    if (item) {
      setTableName(item.FeedbackTableName)
      form.setFieldsValue({ accountName: item.FeedbackAccountName, service: item.MapServerName, layerName: item?.LayerName.split(',') || [] })
    }
  }

  const clear = () => {
    addons.setValue('tableName', '')
    addons.setValue('accountName', '')
    addons.setValue('service', '')
    addons.setValue('layerName', [])
    onChange([])
    setVisible(false)
  }

  const getProgramme = async () => {
    const { code, data } = await GetAreaTaskProgramme()
    if (code === 0) {
      setTaskList(data)
    }
  }

  useEffect(() => {
    if (visible) {
      // if (service && layerName) {
      //   getFromField()
      // }
      // if (tableName) {
      //   getToField()
      // }
      form.setFieldsValue({ schemeName: formData.schemeName || '' })
      getProgramme()
    }
  }, [visible, service, layerName, tableName])

  return (
    <div className={styles.fieldshine}>
      <Input
        value={value && value.length ? JSON.stringify(value) : null}
        onClick={inputClick}
      />
      <Drag
        getContainer={false}
        width='500px'
        title={title}
        bodyStyle={{ maxHeight: 500, overflow: 'auto' }}
        visible={visible}
        footer={
          <div>
            <Button onClick={() => setVisible(false)}>取消</Button>
            {/* <Button onClick={() => clear()}>清空</Button> */}
            <Button type="primary" onClick={() => onOk()}>确定</Button>
          </div>
        }
        onCancel={() => setVisible(false)}
      >
        <Form
          name="dynamic_form_nest_item"
          form={form}
          autoComplete="off"
          labelCol={{ span: 5 }}
        >
          <Form.Item label={title} name='schemeName' rules={[{ required: true, message: `${title}必填`, }]}>
            <Select
              allowClear
              showSearch
              options={taskList.map(v => ({ label: v.Name, value: v.Name }))}
              onChange={taskNameChange}
            />
          </Form.Item>
          <Form.Item label='GIS反馈台账' name='accountName' rules={[{ required: true, message: 'gis反馈台账必填', }]}>
            <Select
              disabled
              showSearch
              options={accountList.map(v => ({ label: v.name, value: v.name }))}
            // onChange={(value) => {
            //   let tableName = accountList.find(v => v.name === value)?.tableName
            //   if (tableName) {
            //     addons.setValue('tableName', tableName)
            //     setTableName(tableName)
            //   }
            //   setAccountName(value)
            // }}
            />
          </Form.Item>
          <Form.Item label='方案名称' name='service' rules={[{ required: true, message: '方案名称必填', },]}>
            <Select disabled options={mapServers} onChange={(value) => setService(value)} />
          </Form.Item>
          <Form.Item
            label='图层名称'
            name='layerName'
            rules={[
              {
                required: true,
                message: '图层名称必填',
              },
            ]}
          >
            <Select
              mode="multiple"
              onFocus={layerNameFocus}
              onChange={(value) => {
                setlayerName(value)
              }}
              disabled
            >
              {
                layers.map(v => <Select.Option value={v.layerName}>{v.layerName}</Select.Option>)
              }
            </Select>
          </Form.Item>
          <Form.List name="field" label='字段映射'>
            {(value, { add, remove }) => (
              <>
                {value.map(({ key, name, ...restField }, index) => (
                  <Space
                    key={key}
                    style={{
                      display: 'flex',
                      marginBottom: '5px',
                      justifyContent: 'center',
                      position: 'relative',
                    }}
                    align="baseline"
                  >
                    <Form.Item
                      {...restField}
                      name={[name, 'fromField']}
                      rules={[
                        {
                          required: true,
                          message: '来源字段必填',
                        }
                      ]}
                    >
                      <TreeSelect
                        showSearch
                        style={{ width: '180px' }}
                        dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                        placeholder="请选择字段名 "
                        allowClear
                        treeDefaultExpandAll
                        showCheckedStrategy
                        onChange={fromFieldChange}
                        disabled={index <= 2}
                      >
                        {fromField.map(s => <TreeNode value={s.fieldName} title={s.fieldAliasName} key={s.fieldName} />)}
                      </TreeSelect>
                    </Form.Item>
                    <div className={styles.centerImg}></div>
                    <Form.Item
                      {...restField}
                      name={[name, 'toField']}
                      rules={[
                        {
                          required: true,
                          message: '目标字段必填',
                        },
                        {
                          validator: (rules, value, callback) => {
                            const { field } = form.getFieldsValue()
                            let toFields = Array.from(new Set([...field.map(v => v.toField)]))
                            if (toFields.length !== field.length) {
                              return Promise.reject('目标字段不能重复！')
                            }
                            return Promise.resolve()
                          }
                        }
                      ]}
                    >
                      <TreeSelect
                        showSearch
                        style={{ width: '180px' }}
                        dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                        placeholder="请选择字段名 "
                        allowClear
                        treeDefaultExpandAll
                        showCheckedStrategy
                        disabled={index <= 2}
                      >
                        {toField.map(s => <TreeNode value={s.name} title={s.alias} key={s.name} />)}
                      </TreeSelect>
                    </Form.Item>
                    <Form.Item
                      hidden={true}
                      {...restField}
                      name={[name, 'layerName']}
                    >
                      <Input />
                    </Form.Item>
                    {index > 2 ? <MinusCircleOutlined onClick={() => remove(name)} /> : null}
                  </Space>
                ))}
                <Form.Item style={{ padding: '0 10px', display: 'none' }}>
                  <Button type="dashed" onClick={() => add()} block icon={<PlusOutlined />}>
                    添加映射
                  </Button>
                </Form.Item>
              </>
            )}
          </Form.List>
        </Form>
      </Drag>
    </div>
  )

}

export default AreaTaskShine
